/* CamacAccess.cc */
/* CAMAC interface class library */
/* Edited by Sanshiro E. on 6 October 1997. */
/* Last updated by Sanshiro E. on 16 April 1999. */

#include "CamacAccess.hh"


TCamacController::TCamacController(void) throw(THardwareException)
{
}

TCamacController::~TCamacController()
{
}


TCamacCrate::TCamacCrate(void)
{
    _CamacController = 0;
}

TCamacCrate::TCamacCrate(TCamacController *Controller)
{
    _CamacController = Controller;
}

TCamacCrate::~TCamacCrate()
{
}

void TCamacCrate::InstallController(TCamacController *Controller)
{
    _CamacController = Controller;    
}

void TCamacCrate::Install(TCamacModule *CamacModule, int StationNumber) throw(THardwareException)
{
    if (_CamacController == 0) {
        throw THardwareException(
            "TCamacCrate::Install()",
            "controller is not installed"
        );
    }

    TCamacModule::TModuleCookie Cookie;
    Cookie.CamacController = _CamacController;
    Cookie.CamacCrate = this;
    Cookie.StationNumber = StationNumber;
    
    CamacModule->Attach(Cookie);
    _ModuleList.push_back(CamacModule);
}

void TCamacCrate::Uninstall(TCamacModule *CamacModule)
{
    CamacModule->Detach();
}

TCamacModule::TCamacModule(void)
{
    _IsAttached = 0;
    _IsLamEnabled = 0;
}

TCamacModule::~TCamacModule()
{
    if (_IsAttached) {
	_CamacCrate->Uninstall(this);
    }
}

int TCamacModule::EnableLam(void) throw(THardwareException)
{
    if (_IsLamEnabled) {
	return 0;
    }

    int Data;
    int Q, X;
    int Status = _CamacController->CamacAction(
        _StationNumber, adLamEnable, fnLamEnable, Data, Q, X
    );

    if (!X) {
	throw THardwareException(
            "TCamacModule::EnableLam()", 
            "LAM enable fault (NoX)"
        );
    }
    if (!Q) {
	throw THardwareException(
            "TCamacModule::EnableLam()", 
            "LAM enable fault (NoQ)"
        );
    }

    _IsLamEnabled = 1;

    return Status;
}

int TCamacModule::DisableLam(void) throw(THardwareException)
{
    if (! _IsLamEnabled) {
	return 0;
    }

    int Data;
    int Q, X;
    int Status = _CamacController->CamacAction(
        _StationNumber, adLamDisable, fnLamDisable, Data, Q, X
    );
    if (!X) {
	throw THardwareException(
            "TCamacModule::DisableLam()", 
            "LAM disable fault (NoX)"
        );
    }
    if (!Q) {
	throw THardwareException(
            "TCamacModule::DisableLam()", 
            "LAM disable fault (NoQ)"
        );
    }

    _IsLamEnabled = 0;

    return Status;
}

int TCamacModule::WaitLam(int TimeOut /* sec */) throw(THardwareException)
{
    if (! _IsLamEnabled) {
	EnableLam();
    }

    int Result = _CamacController->WaitLam(TimeOut);
    if (Result < 0) {
        // timed out
        return 0;
    }
    else {
        return (Result & Bit(_StationNumber));
    }
}

bool TCamacModule::IsLamRequested(void)
{
#if 0   // KEKCC driver has not implemented the method ReadLam() yet.
        return _CamacController->ReadLam() & _StationNumberBit;
#else
	int Data;
	int Q, X;
	_CamacController->CamacAction(_StationNumber, adLamTest, fnLamTest, Data, Q, X);

	return Q;
#endif
}

int TCamacModule::StationNumber(void)
{
    return _StationNumber;
}

int TCamacModule::CamacAction(int Address, int Function, int &Data, int &Q, int &X) 
{
    _CamacController->CamacAction(_StationNumber, Address, Function, Data, Q, X);
    
    return Q;
}

int TCamacModule::CamacAction(int Address, int Function, int &Data) throw(THardwareException)
{
    int Q, X;

    _CamacController->CamacAction(_StationNumber, Address, Function, Data, Q, X);
    if (! X) {
	char Message[256];
	sprintf(Message, "No X-response at station %d.", _StationNumber);
	throw THardwareException("TCamacModule::CamacAction", Message);
    }

    return Q;
}

int TCamacModule::CamacAction(int Address, int Function) throw(THardwareException)
{
    int Data;
    int Q, X;

    _CamacController->CamacAction(_StationNumber, Address, Function, Data, Q, X);
    if (! X) {
	char Message[256];
	sprintf(Message, "No X-response at station %d.", _StationNumber);
	throw THardwareException("TCamacModule::CamacAction", Message);
    }

    return Q;
}

void TCamacModule::Attach(const TModuleCookie &Cookie)
{
    _CamacController = Cookie.CamacController;
    _CamacCrate = Cookie.CamacCrate;
    _StationNumber = Cookie.StationNumber;
    _StationNumberBit = Bit(_StationNumber);
    
    _IsAttached = 1;
}

void  TCamacModule::Detach()
{
    _IsAttached = 0;
}
