/* CamacAccess.hh */
/* CAMAC I/O class library */
/* Edited by Sanshiro E. on 23 October 1997. */
/* Last updated by Sanshiro E. on 16 April 1999. */

#ifndef __CamacAccess_hh__
#define __CamacAccess_hh__


#include <stdio.h>
#include <vector>
#include "RoomDefs.hh"
#include "Module.hh"


class TCamacController {
  public:
    TCamacController(void) throw(THardwareException);
    virtual ~TCamacController();
    virtual int CamacAction(int StationNumber, int Address, int Function, int &Data, int &Q, int &X) throw(THardwareException) = 0;
    virtual int Initialize(void) throw(THardwareException) = 0;
    virtual int Clear(void) throw(THardwareException) = 0;
    virtual int Inhibit(void) throw(THardwareException) = 0;
    virtual int ReleaseInhibit(void) throw(THardwareException) = 0;
    virtual int EnableInterrupt(void) throw(THardwareException) = 0;
    virtual int DisableInterrupt(void) throw(THardwareException) = 0;
    virtual int ReadLam(void) throw(THardwareException) = 0;
    virtual int WaitLam(int TimeOut /* sec */) throw(THardwareException) = 0;
};

class TCamacModule;

class TCamacCrate {
  public:
    TCamacCrate(void);
    TCamacCrate(TCamacController *Controller);
    virtual ~TCamacCrate();
    virtual void InstallController(TCamacController *Controller);
    virtual void Install(TCamacModule *CamacModule, int StationNumber) throw(THardwareException);
    virtual void Uninstall(TCamacModule *CamacModule);
  private:
    TCamacController *_CamacController;
    vector<TCamacModule*> _ModuleList;
};

class TCamacModule: public TModule {
  public:
    TCamacModule(void);
    virtual ~TCamacModule();
    virtual int EnableLam(void) throw(THardwareException);
    virtual int DisableLam(void) throw(THardwareException);
    virtual int WaitLam(int TimeOut /* sec */) throw(THardwareException);
    virtual bool IsLamRequested(void);
    int StationNumber(void);
  protected:
    int CamacAction(int Address, int Function, int &Data, int &Q, int &X);
    int CamacAction(int Address, int Function, int &Data) throw(THardwareException);
    int CamacAction(int Address, int Function) throw(THardwareException);
  private:
    friend void TCamacCrate::Install(TCamacModule *CamacModule, int StationNumber) throw(THardwareException);
    friend void TCamacCrate::Uninstall(TCamacModule *CamacModule);
    struct TModuleCookie{
	TCamacController *CamacController;
	TCamacCrate *CamacCrate;
	int StationNumber;
    };
    void Attach(const TModuleCookie &Cookie);
    void Detach(void);
    int _IsAttached;
  protected:
    enum TFunctionTable {
	fnLamTest = 8,
	fnLamEnable = 26, fnLamDisable = 24
    };
    enum TAddressTable {
	adLamTest = 0,
	adLamEnable = 0, adLamDisable = 0
    };
    TCamacController *_CamacController;
    TCamacCrate *_CamacCrate;
    int _StationNumber;
    unsigned _StationNumberBit;
    bool _IsLamEnabled;
};

#endif
