/* CamacModules.hh */
/* Edited by Sanshiro E. on 26 May 1997. */
/* Last updated by Sanshiro E. on 16 April 1999. */

#ifndef __CamacModules_hh__
#define __CamacModules_hh__

#include <iostream.h>
#include "RoomDefs.hh"
#include "CamacAccess.hh"
#include "Module.hh"



class TCommonplaceCamacModule: public TCamacModule {
  public:
    struct TCamacParameters {
	int NumberOfChannels, NumberOfSamples;
	int ReadFunction, ClearFunction;
    };
  public:
    TCommonplaceCamacModule(TCamacParameters& CamacParameters);
    virtual ~TCommonplaceCamacModule();
    virtual int Read(int Channel, int &Data) throw(THardwareException);
    virtual int Clear(int Channel = -1) throw(THardwareException);
    virtual int HasData(int Channel = -1) throw(THardwareException);
    virtual int WaitData(unsigned TimeOut = 1 /* sec */) throw(THardwareException);
    virtual int NumberOfChannels(void) throw(THardwareException);
    virtual int NumberOfSamples(void) throw(THardwareException);
  protected:
    TCommonplaceCamacModule(void);
    TCamacParameters _CamacParameters;
};


/* Charge ADC Model RPC-022 (Rinei) */
class TCamacQADC_RPC022: public TCommonplaceCamacModule {
  public:
    TCamacQADC_RPC022(void);
  private:
    enum TFunctionTable {
	fnDataRead = 0, fnClear = 9
    };
    enum TQADCParameters {
	_NumberOfChannels = 16, _NumberOfSamples = 1
    };
};

/* Charge ADC Model C009 (Hohshin) */
class TCamacQADC_C009: public TCommonplaceCamacModule {
  public:
    TCamacQADC_C009(void);
  private:
    enum TFunctionTable {
	fnDataRead = 0, fnClear = 9
    };
    enum TQADCParameters {
	_NumberOfChannels = 16, _NumberOfSamples = 1
    };
};

/* TDC Model RPC-060 (Rinei) */
class TCamacTDC_RPC060: public TCommonplaceCamacModule {
  public:
    TCamacTDC_RPC060(void);
  private:
    enum TFunctionTable {
	fnDataRead = 0, fnClear = 9
    };
    enum TTDCParameters {
	_NumberOfChannels = 8, _NumberOfSamples = 1
    };
};

/* TDC Model C006 (Hoshin) */
class TCamacTDC_C006: public TCommonplaceCamacModule {
  public:
    TCamacTDC_C006(void);
  private:
    enum TFunctionTable {
	fnDataRead = 0, fnClear = 9
    };
    enum TTDCParameters {
	_NumberOfChannels = 8, _NumberOfSamples = 1
    };
};

/* Flush ADC MODEL RPC-081 (Rinei) */
class TCamacFADC_RPC081: public TCommonplaceCamacModule {
  public:
    TCamacFADC_RPC081(void);
    virtual int Clear(int Channel) throw(THardwareException);
  private:
    enum TFunctionTable {
	fnDataRead = 0, fnClear = 9
    };
    enum TFADCParameters {
	_NumberOfChannels = 4, _NumberOfSamples = 1024
    };
};


#endif
