/* ModuleFactory.cc */
/* Edited by Sanshiro E. on 7 May 1999. */
/* Last updated by Sanshiro E. on 7 May 1999. */


#include <string>
#include <iostream.h>
#include "VmeModules.hh"
#include "CamacModules.hh"
#include "SoftwareModules.hh"
#include "Controllers.hh"
#include "DeviceFactory.hh"


TDeviceFactory::TDeviceFactory()
{
}

TDeviceFactory::~TDeviceFactory()
{
}

TVmeModule* TDeviceFactory::CreateVmeModule(const char* Name) throw(THardwareException)
{
    string ModuleName(Name);
    TVmeModule* Module = 0;

    if (ModuleName == "Rinei_RPV130") {
        Module = new TVmeIORegister_RPV130();
    }
    else if (ModuleName == "Hoshin_V004") {
        Module = new TVmeScaler_V004();
    }
    else {
        throw THardwareException(
            "TDeviceFactory::CreateVmeModule()",
            "No such vme module"
        );
    }

    return Module;
}

TCamacModule* TDeviceFactory::CreateCamacModule(const char* Name) throw(THardwareException)
{
    string ModuleName(Name);
    TCamacModule* Module = 0;

    if (ModuleName == "Rinei_RPC022") {
        Module = new TCamacQADC_RPC022();
    }
    else if (ModuleName == "Hoshin_C009") {
        Module = new TCamacQADC_C009();
    }
    else if (ModuleName == "Rinei_RPC060") {
        Module = new TCamacTDC_RPC060();
    }
    else if (ModuleName == "Hoshin_C006") {
        Module = new TCamacTDC_C006();
    }
    else if (ModuleName == "Rinei_RPC081") {
        Module = new TCamacFADC_RPC081();
    }

    else {
        throw THardwareException(
            "TDeviceFactory::CreateCamacModule()",
            "No such camac module"
        );
    }

    return Module;
}

TSoftwareModule* TDeviceFactory::CreateSoftwareModule(const char* Name) throw(THardwareException)
{
    string ModuleName(Name);
    TSoftwareModule* Module = 0;

    if (ModuleName == "PoissonGenerator") {
        Module = new TPoissonGenerator();
    }
    else if (ModuleName == "SoftwareCounter") {
        Module = new TSoftwareCounter();
    }

    else {
        throw THardwareException(
            "TDeviceFactory::CreateSoftwareModule()",
            "No such software module"
        );
    }

    return Module;
}

TVmeCrate* TDeviceFactory::CreateVmeCrate(const char* Name) throw(THardwareException)
{
    return new TVmeCrate();
}

TCamacCrate* TDeviceFactory::CreateCamacCrate(const char* Name) throw(THardwareException)
{
    return new TCamacCrate();
}

TVmeController* TDeviceFactory::CreateVmeController(const char* Name) throw(THardwareException)
{
    string ControllerName(Name);
    TVmeController* Controller = 0;

    if (0) {
    }
#ifdef USE_FRCVME
    else if ((Name == 0) || (string(Name) == "FORCE_FGA5000")) {
        Controller = new TVmeController_FGA5000();
    }
#endif
    else {
        throw THardwareException(
            "TDeviceFactory::CreateVmeController()",
            "No such vme controller"
        );
    }

    return Controller;
}

TCamacController* TDeviceFactory::CreateCamacController(const char* Name) throw(THardwareException)
{
    TCamacController* Controller = 0;

    if (0) {
    }
#ifdef USE_KEKCC
    else if ((Name == 0) || (string(Name) == "KEK_CC")) {
        Controller = new TCamacController_KEKCC();
    }
#endif
#ifdef USE_CAMDRV
    else if ((Name == 0) || (string(Name) == "TOYO_CC7700")) {
        Controller = new TCamacController_CC7700();
    }
#endif
    else {
        throw THardwareException(
            "TDeviceFactory::CreateCamacController()",
            "No such camac controller"
        );
    }

    return Controller;
}
