/* FORCE_FGA5000.cc */
/* VME interface class library for FORCE FGA-5000 */
/* Edited by Sanshiro E. on 6 October 1997. */
/* Last updated by Sanshiro E. on 1 March 1999. */


#include <stdio.h>
#include <iostream.h>
#include <string.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <sys/vme_types.h>
#include <sys/vme.h>
#include "vui.hh"
#include "FORCE_FGA5000.hh"


static const char *VmeDeviceFileName[] = {
    /* this must be consistent with TVmeTarnsferMode (declared in VmeAccess.hh) */
    "/dev/vme16d16", "/dev/vme16d32",
    "/dev/vme24d16", "/dev/vme24d32", 
    "/dev/vme32d16", "/dev/vme32d32"
};

TVmeController_FGA5000::TVmeController_FGA5000(void)
{
}

TVmeController_FGA5000::~TVmeController_FGA5000()
{
}

int TVmeController_FGA5000::Open(TVmeTransferMode TransferMode) throw(THardwareException)
{
    int DeviceDescripter = open(VmeDeviceFileName[TransferMode], O_RDWR);
    if (DeviceDescripter < 0) {
	throw THardwareException("TVmeController_FGA5000::Open(): open(2)", strerror(errno));
    }
    
    return DeviceDescripter;
}

void TVmeController_FGA5000::Close(int DeviceDescripter)
{
    close(DeviceDescripter);
}

caddr_t TVmeController_FGA5000::Map(int DeviceDescripter, off_t VmeAddress, size_t MapSize) throw(THardwareException)
{
    caddr_t MapAddress = (caddr_t) 0;   // MapAddress will be assigned by kernel.
    int MapProtection = PROT_READ | PROT_WRITE;
    int MapFlag = MAP_SHARED;
    caddr_t MappedAddress = mmap(
	MapAddress, MapSize, MapProtection, MapFlag, DeviceDescripter, VmeAddress
    );
    if ((int) MappedAddress == -1) {
	throw THardwareException("TVmeController_FGA5000::Map: mmap(2)", strerror(errno));
    }
    
    return MappedAddress;
}

void TVmeController_FGA5000::Unmap(caddr_t MappedAddress, size_t MapSize) throw(THardwareException)
{
    munmap(MappedAddress, MapSize);
}

void TVmeController_FGA5000::EnableInterrupt(int DeviceDescripter, int InterruptNumber, int SignalID) throw(THardwareException)
{
    ioctl_irq_t InterruptDefinition;
    InterruptDefinition.prop = InterruptNumber;
    InterruptDefinition.sig = SignalID;
    InterruptDefinition.level = 3;     // not relevant for this case
    InterruptDefinition.vect = 0xf0;   // not relevant for this case
  
    if (vui_intr_ena(DeviceDescripter, &InterruptDefinition) != VUI_OK) {
	throw THardwareException(
	    "TVmeModule::EnableInterrupt(): ", 
	    "interrupt enable fault.\n"
	    "  1) Doesn't another process use same interrupt?\n"
	    "  2) Is interrupt property(in /kernel/drv/vmeplus.conf) set correctly?"
        );	
    }
}

void TVmeController_FGA5000::DisableInterrupt(int DeviceDescripter, int InterruptNumber, int SignalID) throw(THardwareException)
{
    ioctl_irq_t InterruptDefinition;
    InterruptDefinition.prop = InterruptNumber;
    InterruptDefinition.sig = SignalID;

    if (vui_intr_dis(DeviceDescripter, &InterruptDefinition) != VUI_OK) {
	throw THardwareException(
	    "TVmeModule::EnableInterrupt(): ", 
	    "interrupt disable fault.\n"
	    "  1) Doesn't another process use same interrupt?\n"
	    "  2) Is interrupt property(in /kernel/drv/vmeplus.conf) set correctly?"
        );	
    }
}

