/* Module.cc */
/* kinoko standard interface for DAQ modules */
/* Edited by Sanshiro E. on 10 July 1998. */
/* Last updated by Sanshiro E. on 3 March 1999. */


#include <iostream.h>
#include "Module.hh"


TModule::TModule(void)
{
}

TModule::~TModule()
{
}

int TModule::ModuleAddress(void)  throw(THardwareException)
{
    throw THardwareException(
        "TModule::ModuleAddress()", 
	"Inappropriate function called (internal error)"
    );

    return 0;
}

int TModule::Initialize(int InitialState) throw(THardwareException)
{
    return 0;
}

int TModule::Finalize(int FinalState) throw(THardwareException)
{
    return 0;
}

int TModule::Start(int Channel) throw(THardwareException)
{
    throw THardwareException(
        "TModule::Start()", 
	"This function is not supported for the module."
    );

    return 0;
}

int TModule::Stop(int Channel) throw(THardwareException)
{
    throw THardwareException(
        "TModule::Stop()", 
	"This function is not supported for the module."
    );

    return 0;
}

int TModule::IsRunning(int Channel) throw(THardwareException)
{
    throw THardwareException(
        "TModule::IsRunning()", 
	"This function is not supported for the module."
    );

    return 0;
}

int TModule::IsBusy(int Channel) throw(THardwareException)
{
    throw THardwareException(
        "TModule::IsBusy()", 
	"This function is not supported for the module."
    );

    return 0;
}

int TModule::Clear(int Channel) throw(THardwareException)
{
    return 0;
}

int TModule::Read(int Channel, int &Data)  throw(THardwareException)
{
    throw THardwareException(
        "TModule::Read()", 
	"This function is not supported for the module."
    );

    return 0;
}

int TModule::Write(int Channel, int Data)  throw(THardwareException)
{
    throw THardwareException(
        "TModule::Write()", 
	"This function is not supported for the module."
    );

    return 0;
}

int TModule::BlockRead(int Channel, void *Data, int MaxSize)  throw(THardwareException)
{
    throw THardwareException(
        "TModule::BlockRead()", 
	"This function is not supported for the module."
    );

    return 0;
}

int TModule::BlockWrite(int Channel, const void *Data, int Size)  throw(THardwareException)
{
    throw THardwareException(
        "TModule::BlockWrite()", 
	"This function is not supported for the module."
    );

    return 0;
}

int TModule::SequencialRead(int Channel, int Data[], int MaxSize) throw(THardwareException)
{
    int DataCount = 0;

    while ((DataCount < MaxSize) && (Read(Channel, Data[DataCount]) > 0)) {
	DataCount++;
    }

    return DataCount;
}

int TModule::SequencialWrite(int Channel, int Data[], int Size) throw(THardwareException)
{
    int DataCount = 0;

    while ((DataCount < Size) && (Write(Channel, Data[DataCount]) > 0)) {
	DataCount++;
    }

    return DataCount;
}

int TModule::WaitData(unsigned TimeLimit) throw(THardwareException)
{
    unsigned Time = 0;
    int EventOccured;

    while (1) {
	EventOccured = HasData();
	if ((EventOccured) || (Time > TimeLimit)) {
	    break;
	}
	
	if (TimeLimit > 0) {
	    Time++;  // sec
	}
	Suspend(1, 0);
    }

    return EventOccured;
}

int TModule::HasData(int Channel) throw(THardwareException)
{
    throw THardwareException(
        "TModule::HasData()", 
	"This function is not supported for the module."
    );

    return 0;
}

int TModule::GetData(int Channel) throw(THardwareException)
{
    int Data;

    if (Read(Channel, Data) != 1) {
	throw THardwareException("TModule::GetData()", "can't get data.");
    }

    return Data;
}

int TModule::NumberOfChannels(void) throw(THardwareException)
{
    throw THardwareException(
        "TModule::NumberOfChannels()", 
	"This function is not supported for the module."
    );

    return 0;
}

int TModule::NumberOfSamples(void) throw(THardwareException)
{
    return 1;
}

#if 0
int TModule::EnableInterrupt(void) throw(THardwareException)
{
    throw THardwareException(
        "TModule::EnableInterrupt()", 
	"This function is not supported for the module."
    );

    return 0;
}

int TModule::DisableInterrupt(void) throw(THardwareException)
{
    throw THardwareException(
        "TModule::DisableInterrupt()", 
	"This function is not supported for the module."
    );

    return 0;
}

int TModule::InstallInterruptHandler(TInterruptHandler *InterruptHandler)
{
    _InterruptHandler = InterruptHandler;

    return 0;
}
#endif
