/* Module.hh */
/* kinoko standard interface for hardware modules */
/* Edited by Sanshiro E. on 10 July 1998. */
/* Last updated by Sanshiro E. on 3 March 1999. */

#ifndef __module_hh__
#define __module_hh__

#include <iostream.h>
#include "RoomDefs.hh"


class TInterruptHandler;

class TModule {
  protected:
    TModule(void);
  public:
    virtual ~TModule();
    virtual int ModuleAddress(void) throw(THardwareException);
    virtual int Initialize(int InitialState = 0) throw(THardwareException);
    virtual int Finalize(int FinalState = 0) throw(THardwareException);
    virtual int Start(int Channel = -1) throw(THardwareException);
    virtual int Stop(int Channel = -1) throw(THardwareException);
    virtual int IsRunning(int Channel = -1) throw(THardwareException);
    virtual int IsBusy(int Channel = -1) throw(THardwareException);
    virtual int Clear(int Channel = -1) throw(THardwareException);
    virtual int Read(int Channel, int &Data) throw(THardwareException);
    virtual int Write(int Channel, int Data) throw(THardwareException);
    virtual int BlockRead(int Channel, void *Data, int MaxSize) throw(THardwareException);
    virtual int BlockWrite(int Channel, const void *Data, int Size) throw(THardwareException);
    virtual int SequencialRead(int Channel, int Data[], int MaxSize) throw(THardwareException);
    virtual int SequencialWrite(int Channel, int Data[], int Size) throw(THardwareException);
    virtual int WaitData(unsigned TimeLimit = 0 /* sec */) throw(THardwareException);
    virtual int HasData(int Channel = -1) throw(THardwareException);
    virtual int GetData(int Channel) throw(THardwareException);
    virtual int NumberOfChannels(void) throw(THardwareException);
    virtual int NumberOfSamples(void) throw(THardwareException);
#if 0
    virtual int EnableInterrupt(void) throw(THardwareException);
    virtual int DisableInterrupt(void) throw(THardwareException);
    virtual int InstallInterruptHandler(TInterruptHandler *InterruptHandler);
#endif
  protected:
    TInterruptHandler *_InterruptHandler;
};

class TInterruptHandler {
  public:
    TInterruptHandler(void) {}
    virtual ~TInterruptHandler() {}
    virtual void OnInterrupt(TModule *Module) {}
};


#endif
