/* SoftwareModule.cc */
/* kinoko software module */
/* Edited by Sanshiro E. on 23 April 1998. */
/* Last updated by Sanshiro E. on 27 April 1999. */


#include <iostream.h>
#include <math.h>
#include <stdlib.h>
#include "SoftwareModules.hh"


static double Poisson(double Lambda);


TPoissonGenerator::TPoissonGenerator(int NumberOfChannels, int NumberOfSamples, long FullScale)
{
    _NumberOfChannels = NumberOfChannels;
    _NumberOfSamples = NumberOfSamples;
    _FullScale = FullScale;

    _ReadoutCount = new int[NumberOfChannels];
    Clear();
}

TPoissonGenerator:: ~TPoissonGenerator()
{
    delete _ReadoutCount;
}
    
int TPoissonGenerator::Clear(int Channel) throw(THardwareException)
{
    if (Channel < 0) {
	for (int Channel = 0; Channel < _NumberOfChannels; Channel++) {
	    _ReadoutCount[Channel] = 0;
	}
    }
    else if (Channel < _NumberOfChannels) {
	_ReadoutCount[Channel] = 0;
    }
    else {
	return -1;
    }

    return 0;
}

int TPoissonGenerator::Read(int Channel, int &Data) throw(THardwareException)
{
    if (_ReadoutCount[Channel] >= _NumberOfSamples) {
	return 0;
    }
    else {
	_ReadoutCount[Channel]++;
    }
    
    double Lambda = 10;
    Data = (int) Poisson(Lambda) * (_FullScale / (3.0 * Lambda));

    if (Data > _FullScale) {
	Data = _FullScale;
    }

    return 1;
}

int TPoissonGenerator::NumberOfChannels(void) throw(THardwareException)
{
    return _NumberOfChannels;
}

int TPoissonGenerator::NumberOfSamples(void) throw(THardwareException)
{
    return _NumberOfSamples;
}

static double Poisson(double Lambda)
{
    // Poisson distribution
    // P(k) = exp(-Lambda) Lambda^k / k!

    Lambda = exp(Lambda) * drand48();
    int k = 0;
    while (Lambda > 1) {
	Lambda *= drand48();
	k++;
    }

    return k;
}


TSoftwareCounter::TSoftwareCounter(int NumberOfChannels, int NumberOfSamples, long FullScale)
{
    _NumberOfChannels = NumberOfChannels;
    _NumberOfSamples = NumberOfSamples;
    _FullScale = FullScale;

    _ReadoutCount = new int[NumberOfChannels];
    Initialize();
}

TSoftwareCounter:: ~TSoftwareCounter()
{
    delete _ReadoutCount;
}
    
int TSoftwareCounter::Initialize(int InitialState) throw(THardwareException)
{
    for (int Channel = 0; Channel < _NumberOfChannels; Channel++) {
	_ReadoutCount[Channel] = 0;
    }

    return 0;
}

int TSoftwareCounter::Read(int Channel, int &Data) throw(THardwareException)
{
    Data = _ReadoutCount[Channel];
    
    if (Data >= _FullScale) {
	Data = _FullScale;
    }
    else {
	_ReadoutCount[Channel]++;
    }

    return 1;
}

int TSoftwareCounter::NumberOfChannels(void) throw(THardwareException)
{
    return _NumberOfChannels;
}

int TSoftwareCounter::NumberOfSamples(void) throw(THardwareException)
{
    return _NumberOfSamples;
}
