/* TMC.cc */
/* Edited by Sanshiro E. on 26 May 1997. */
/* Last updated by Sanshiro E. on 16 June 1998. */

#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>
#include "modules.hh"


TTMC_32chVME::TTMC_32chVME(void)
:TVmeModule((size_t) MemorySize, (TVmeTransferMode) TransferMode)
{
}

void TTMC_32chVME::Initialize(void) throw(THardwareException)
{
    PutWord(drun, VME_not_ready);
    PutWord(dcstsp, common_stop_mode);
    PutWord(dcount, 100);
    PutWord(ddisp, all_channel_data);
    PutWord(dsuboff, donot_substruct);
    PutWord(dedge, both);
    PutWord(dmodule, (ModuleAddress() >> 16) & 0x00FF);
    PutWord(dready, buffer_empty);

    PutWord(drun, VME_ready);
    
    unsigned TimeLimit = 1000 /* ms */;
    if (WaitWord(dready, data_exist, TimeLimit) == wrTimeOut) {
	throw THardwareException("TTMC_32chVME::Initialize()", "Initialization timeout");
    }
}

void TTMC_32chVME::Clear(void)
{
    PutWord(dready, buffer_empty);
}

void TTMC_32chVME::WaitData(void)
{
    WaitWord(dready, data_exist);
}

void TTMC_32chVME::Read(int &NumberOfData, TDataRecord *(&DataRecord)) throw(THardwareException)
{
    NumberOfData = (GetWord(ntotal) - (begin_of_data - ntotal) / 2) / 2;
    DataRecord = (TDataRecord *) WordPtr(begin_of_data);

    if (*((Word *) (DataRecord + NumberOfData)) != 0x5555) {
	throw THardwareException("TTMC_32chVME::Read()", "data inconsistent");
    }
}

void TTMC_32chVME::TDataDecorder::SetDataRecord(TDataRecord *DataRecord)
{
    _DataRecord = DataRecord;
}
	
TTMC_32chVME::TDataDecorder &TTMC_32chVME::TDataDecorder::GoNext(void)
{
    _DataRecord++;
    return *this;
}

int TTMC_32chVME::TDataDecorder::IsOnEndOfData(void)
{
    return (* (Word *) _DataRecord == 0x5555);
}

int TTMC_32chVME::TDataDecorder::ModuleID(void) 
{
    return (_DataRecord->Address >> moduleID_shift);
}

int TTMC_32chVME::TDataDecorder::ChannelNo(void) 
{
    return (_DataRecord->Address & channelNo_mask);
}

char TTMC_32chVME::TDataDecorder::DetectedEdge(void) 
{
    if ((_DataRecord->Data >> edge_shift) == rising_edge_data) {
	return 'R';
    }
    else {
	return 'F';
    }
}

int TTMC_32chVME::TDataDecorder::Data(void) 
{
    return (_DataRecord->Data & data_mask);
}



TTMC_32chCAMAC::TTMC_32chCAMAC(void) 
{
}

TTMC_32chCAMAC::~TTMC_32chCAMAC() 
{
}

void TTMC_32chCAMAC::Initialize(void) 
{
    CamacAction(adNone, fnReset);
}

void TTMC_32chCAMAC::Clear(void) 
{
    CamacAction(adNone, fnStart);
}

void TTMC_32chCAMAC::Read(int &NumberOfData, TDataRecord *(&DataRecord)) 
{
    static TDataRecord Buffer[_NumberOfColumns * _NumberOfChannels];
    Word StopTime[_NumberOfChannels] = {0};
    NumberOfData = 0;

    for (unsigned ChipNumber = 0; ChipNumber < _NumberOfChips; ChipNumber++) {
	int ReadPoint, WritePoint, Data;
	CamacAction(ChipNumber, fnReadCSR2, WritePoint);

	/* read input signal */
	int LastRowHasSignal[_NumberOfChannelsPerChip] = {0};
	int Row;
	for (Row = 0; Row < _NumberOfRows - 2; Row++) {
	    ReadPoint = WritePoint - _NumberOfRows + Row;
	    CamacAction(ChipNumber, fnWriteCSR1, ReadPoint);

	    for (unsigned Channel = 0; Channel < _NumberOfChannelsPerChip; Channel++) {
		CamacAction(ChipNumber, fnDataRead, Data);
		Data = (Data >> ((Channel % 4) * 6)) & 0x003f;	 // extract one channel 
fprintf(stderr, "%02d-%02d,%02d: %02x\n", ChipNumber, Channel, Row, Data);

		if (LastRowHasSignal[Channel]) {
		    if (Data & continue_bit) {
			continue;
		    }
		    else {
			LastRowHasSignal[Channel] = 0;
		    }
		}

		if (Data) {
		    if (Data & continue_bit) {
			Data = 0;
		    }
		    else {
			Data &= data_mask;
		    }

		    Word Address = ChipNumber * _NumberOfChannelsPerChip + Channel;
		    Buffer[NumberOfData].Address = Address;
		    Buffer[NumberOfData].Data = Row * _NumberOfColumns + Data;
fprintf(stdout, "\tTMC[%02d][%02d] = %u\n", StationNumber(), Address, Buffer[NumberOfData].Data);
		    NumberOfData++;

		    LastRowHasSignal[Channel] = 1;
		}
	    }
	}

	/* read stop signal */
	int StopMarkCount = 0;
	for ( ; Row < _NumberOfRows; Row++) {
	    ReadPoint = WritePoint - _NumberOfRows + Row;
	    CamacAction(ChipNumber, fnWriteCSR1, ReadPoint);

	    for (unsigned Channel = 0; Channel < _NumberOfChannelsPerChip; Channel++) {
		Word Address = ChipNumber * _NumberOfChannelsPerChip + Channel;
		if (StopTime[Address] != 0) {
		    continue;
		}

		CamacAction(ChipNumber, fnDataRead, Data);
		Data = (Data >> ((Channel % 4) * 6)) & 0x003f;	 // extract one channel 
fprintf(stderr, "%02d-%02d,%02d: %02x\n", ChipNumber, Channel, Row, Data);

		if (Data) {
		    if (Data & continue_bit) {
			Data = 0;
		    }
		    else {
			Data &= data_mask;
		    }

		    StopTime[Address] = Row * _NumberOfColumns + Data;
fprintf(stdout, "\tStopTime[%02d][%02d] = %u\n", StationNumber(), Address, StopTime[Address]);
		    StopMarkCount++;
		}
	    }
	}
	if (StopMarkCount != _NumberOfChannelsPerChip) {
	    //throw THardwareException("TTMC_32chCAMAC::Read", "no stop mark");
	}
    }
    
    /* substruct stop time */
    for (int i = 0; i < NumberOfData; i++) {
	Buffer[i].Data = StopTime[Buffer[i].Address] - Buffer[i].Data;
    }

    DataRecord = Buffer;
}


