/* TMC.hh */
/* Edited by Sanshiro E. on 26 May 1997. */
/* Last updated by Sanshiro E. on 16 June 1998. */

#ifndef __TMC_hh__
#define __TMC_hh__

#include <iostream.h>
#include "RoomDefs.hh"
#include "VMEslib.hh"
#include "CAMslib.hh"


/* 32ch TMC-VME Module */
class TTMC_32chVME: public TVmeModule {
  public:
    struct TDataRecord {
	Word Address;
	Word Data;
    };
  public:
    TTMC_32chVME(void);
    virtual ~TTMC_32chVME() {}
    void Initialize(void) throw(THardwareException);
    void Clear(void);
    void WaitData(void);
    void Read(int &NumberOfData, TDataRecord *(&DataRecord)) throw(THardwareException);
  public:
    class TDataDecorder {
      public:
	void SetDataRecord(TDataRecord *DataRecord);
	TDataDecorder &GoNext(void);
	int IsOnEndOfData(void);
	int ModuleID(void);
	int ChannelNo(void);
	char DetectedEdge(void);
	int Data(void);
      protected:
	TDataRecord *_DataRecord;
    };
    friend class TDataDecorder;
  protected:
    enum TTMCParameters {
	MemorySize = 0x4000,
	TransferMode = tmA24D16
    };
    enum TTMCSymbols {
	dready = 0x0000,  // Data ready flag
	    buffer_empty = 0,
	    data_exist = 1,
	ntotal = 0x0020,  // Total number of word (16bit word)
	status = 0x0022,  // Status flag
	    no_rec_stsp = 01,
	    overflow = 02,
	    no_eod = 04,
	t0 = 0x0024,  // Common start/stop time
	stsp_time = 0x0024,
	lastwp = 0x0026,  // Last write pointer value of TMC chip
	begin_of_data = 0x0028,
	    moduleID_shift = 5,
	    channelNo_mask = 0x001F,
	    edge_shift = 15,  // Falling edge/Rising edge data flag
		rising_edge_data = 0,
		falling_edge_data = 1,
	    data_mask = 0x0FFF,
	drun = 0x3f00,  // DAQ run flag
	    VME_not_ready = 0,
	    VME_ready = 1,
	dcstsp = 0x3f02,  // Common start/stop mode
	    common_stop_mode = 0,
	    common_start_mode = 1,
	dcount = 0x3f04,  // Time range count (Recording time = 32ns * dcount)
	ddisp = 0x3f06,  // Serial port display flag
	    error_message = 98,
	    all_channel_data = 99,
	    no_display = 100,
	dsuboff = 0x3f08,  // Substruct offset flag  (NOT SUPPORTED NOW)
	    default_value = 0,
	    donot_substruct = 1,
	    offset_value = 2,
	dedge = 0x3f0a,  // Edge detection flag
	    both = 0,
	    rising = 1,
	    falling = 2,
	    both_but_1st_falling = 3,
	dmodule = 0x3f0c,  // Module ID appended to the channel number
	doffset = 0x3f10  // Channel offset numbers  (NOT SUPPORTED NOW)
    };
};

/* 32ch TMC-CAMAC Module */
/* UNCOMPLETE!! */
class TTMC_32chCAMAC: public TCamacModule {
  public:
    struct TDataRecord {
	Word Address;
	Word Data;
    };
  public:
    TTMC_32chCAMAC(void);
    virtual ~TTMC_32chCAMAC();
    void Initialize(void);
    virtual void Clear(void);
    void Read(int &NumberOfData, TDataRecord *(&DataRecord));
  protected:
    enum TAddressTable {
	adNone = 0
    };
    enum TFunctionTable {
	fnDataRead = 0, fnReset = 9, 
	fnReadCSR0 = 1, fnReadCSR1 = 4, fnReadCSR2 = 6,
	fnWriteCSR0 = 17, fnWriteCSR1 = 20, fnWriteCSR2 = 22,
	fnStart = 25
    };
    enum TTMCParameters {
	_NumberOfChannels = 32, _NumberOfChannelsPerChip = 4,
	_NumberOfRows = 32, _NumberOfColumns = 32, _NumberOfChips = 8
    };
    enum TTMCSymbol {
	data_mask = 0x001f,
	continue_bit = 0x0020
     };
};

#endif
