/* TOYO_CC7700.cc */
/* CAMAC I/O class library for CC/7700 (camdrv) */
/* Edited by Sanshiro E. on 13 April 1999. */
/* Last updated by Sanshiro E. on 13 April 1999. */

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include "camdrv.h"
#include "TOYO_CC7700.hh"


#define NAF(n, a, f) ((((n) << 9) | ((a) << 5) | (f)) & 0x00003fff)


TCamacController_CC7700::TCamacController_CC7700(void) throw(THardwareException)
{
    const char* DeviceFile = "/dev/camdrv";
    _Device = open(DeviceFile, O_RDWR);
    if (_Device < 0) {
	throw THardwareException(
	   "TCamacController_CC7700::Open(): open(2)",
           strerror(errno)
        );
    }

    _IsInterruptEnabled = 0;
}

TCamacController_CC7700::~TCamacController_CC7700()
{
    if (_IsInterruptEnabled) {
	DisableInterrupt();
    }

    if (_Device > 0) {
        close(_Device);
    }
}

int TCamacController_CC7700::CamacAction(int StationNumber, int Address, int Function, int &Data, int &Q, int &X) throw(THardwareException)
{
    static unsigned IoctlData[2];

    IoctlData[0] = NAF(StationNumber, Address, Function);
    IoctlData[1] = (unsigned) Data;

    int Result = ioctl(_Device, CAMDRV_IOC_CAMAC_ACTION, IoctlData);
    if (Result < 0) {
	throw THardwareException(
	   "TCamacController_CC7700::CamacAction(): ioctl(2)", 
           strerror(errno)
        );
    }
        
    Data = IoctlData[1] & 0x00FFFFFF;
    X = ! (Result & 0x0002);
    Q = ! (Result & 0x0001);

    //    cerr << Result << endl;

    return Result;
}

int TCamacController_CC7700::Initialize(void) throw(THardwareException)
{
    int Result = ioctl(_Device, CAMDRV_IOC_INITIALIZE, 0);
    return Result;
}

int TCamacController_CC7700::Clear(void) throw(THardwareException)
{
    int Result = ioctl(_Device, CAMDRV_IOC_CLEAR, 0);
    return Result;
}

int TCamacController_CC7700::Inhibit(void) throw(THardwareException)
{
    int Result = ioctl(_Device, CAMDRV_IOC_INHIBIT, 0);    
    return Result;
}

int TCamacController_CC7700::ReleaseInhibit(void) throw(THardwareException)
{
    int Result = ioctl(_Device, CAMDRV_IOC_RELEASE_INHIBIT, 0);
    return Result;
}

int TCamacController_CC7700::EnableInterrupt(void) throw(THardwareException)
{
    if (_IsInterruptEnabled) {
	return 0;
    }

    int Result = ioctl(_Device, CAMDRV_IOC_ENABLE_INTERRUPT, 0);

    if (Result == 0) {
        _IsInterruptEnabled = 1;
    }

    return Result;
}

int TCamacController_CC7700::DisableInterrupt(void) throw(THardwareException)
{
    if (! _IsInterruptEnabled) {
	return 0;
    }

    int Result = ioctl(_Device, CAMDRV_IOC_DISABLE_INTERRUPT, 0);

    if (Result == 0) {
        _IsInterruptEnabled = 0;
    }

    return Result;
}

int TCamacController_CC7700::ReadLam(void) throw(THardwareException)
{
    static unsigned IoctlData[2];
    IoctlData[0] = 0;
    IoctlData[1] = 0;

    int Result = ioctl(_Device, CAMDRV_IOC_READ_LAM, IoctlData);
    if (Result < 0) {
	throw THardwareException(
           "TCamacController_CC7700::ReadLam(): ioctl(2)",
           strerror(errno)
        );
    }

    int LamBits = IoctlData[1];

    return LamBits;
}

int TCamacController_CC7700::WaitLam(int TimeOut /* sec */) throw(THardwareException)
{
    if (! _IsInterruptEnabled) {
	EnableInterrupt();
    }

    static unsigned IoctlData[2];
    IoctlData[0] = TimeOut;
    IoctlData[1] = 0;

    int Result = ioctl(_Device, CAMDRV_IOC_WAIT_LAM, IoctlData);

    return Result;
}
