/* VmeAccess.cc */
/* VME interface class library */
/* Edited by Sanshiro E. on 6 October 1997. */
/* Last updated by Sanshiro E. on 16 April 1999. */


#include <stdio.h>
#include <iostream.h>
#include <string.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include "VmeAccess.hh"


TVmeController::TVmeController(void)
{
}

TVmeController::~TVmeController()
{
}

TVmeCrate::TVmeCrate(TVmeController *VmeController, int NumberOfSlots) 
{
    _VmeController = VmeController;
    _NumberOfSlots = NumberOfSlots;
}

TVmeCrate::TVmeCrate(int NumberOfSlots) 
{
    _VmeController = 0;
    _NumberOfSlots = NumberOfSlots;
}

TVmeCrate::~TVmeCrate() 
{
}

void TVmeCrate::InstallController(TVmeController *VmeController)
{
    _VmeController = VmeController;
}

void TVmeCrate::Install(TVmeModule *VmeModule, off_t VmeAddress, int InterruptPropertyIndex) throw(THardwareException)
{
    if (_VmeController == 0) {
        throw THardwareException(
            "TVmeCrate::Install()",
            "controller is not installed"
        );
    }

    TVmeTransferMode TransferMode = VmeModule->TransferMode();
    size_t MapSize = VmeModule->MapSize();

    int DeviceDescripter = _VmeController->Open(TransferMode);
    caddr_t MappedAddress = _VmeController->Map(DeviceDescripter, VmeAddress, MapSize);

    /* write cookie */
    TVmeModule::TModuleCookie Cookie;
    Cookie.VmeController = _VmeController;
    Cookie.VmeCrate = this;
    Cookie.DeviceDescripter = DeviceDescripter;
    Cookie.MappedAddress = MappedAddress;
    Cookie.VmeAddress = VmeAddress;
    Cookie.InterruptPropertyIndex = InterruptPropertyIndex;
    VmeModule->Attach(Cookie);

    _ModuleList.push_back(VmeModule);
}

void TVmeCrate::Uninstall(TVmeModule *VmeModule) 
{
    caddr_t MappedAddress = VmeModule->MappedAddress();
    size_t MapSize = VmeModule->MapSize();
    int DeviceDescripter = VmeModule->DeviceDescripter();

    _VmeController->Unmap(MappedAddress, MapSize);
    _VmeController->Close(DeviceDescripter);
}

TVmeModule::TVmeModule(size_t MapSize, TVmeTransferMode TransferMode)
{
    _MapSize = MapSize;
    _TransferMode = TransferMode;
    _IsAttached = 0;
    _InterruptSignalID = 0;
}

TVmeModule::~TVmeModule()
{
    if (_IsAttached) {
	_VmeCrate->Uninstall(this);
    }
}

size_t TVmeModule::MapSize(void) const
{
    return _MapSize;
}

TVmeTransferMode TVmeModule::TransferMode(void) const
{
    return _TransferMode;
}

off_t TVmeModule::VmeAddress(void) const
{
    return _VmeAddress;
}

int TVmeModule::DeviceDescripter(void) const
{
    return _DeviceDescripter;
}

caddr_t TVmeModule::MappedAddress(void) const
{
    return _MappedAddress;
}

int TVmeModule::WaitWord(off_t OffsetAddress, Word Value, unsigned TimeLimit)
{
    unsigned Time = 0;
    int EventOccured;

    while (1) {
	EventOccured = (GetWord(OffsetAddress) == Value);
	if ((EventOccured) || (Time > TimeLimit)) {
	    break;
	}
	
	if (TimeLimit > 0) {
	    Time++;  // sec
	}
	Suspend(1, 0);
    }

    return EventOccured;
}

void TVmeModule::Attach(const TModuleCookie &Cookie)
{
    _VmeController = Cookie.VmeController;
    _VmeCrate = Cookie.VmeCrate;
    _DeviceDescripter = Cookie.DeviceDescripter;
    _VmeAddress = Cookie.VmeAddress;
    _MappedAddress = Cookie.MappedAddress;
    _InterruptPropertyIndex = Cookie.InterruptPropertyIndex;
    _IsAttached = 1;
}

void TVmeModule::Detach(void)
{
    _IsAttached = 0;
}

void TVmeModule::EnableInterrupt(int SignalID) throw(THardwareException)
{
    if (_InterruptPropertyIndex < 0) {
	throw THardwareException(
	    "TVmeModule::EnableInterrupt()", 
	    "interrupt property is not registered."
        );
    }

    _VmeController->EnableInterrupt(_DeviceDescripter, _InterruptPropertyIndex, SignalID);

    _InterruptSignalID = SignalID;
}

void TVmeModule::DisableInterrupt(void) throw(THardwareException)
{
    if (_InterruptSignalID < 0) {
	throw THardwareException(
	    "TVmeModule::DisableInterrupt()", 
	    "the interrupt is not enabled."
        );
    }
    int SignalID = _InterruptSignalID;

    _VmeController->DisableInterrupt(_DeviceDescripter, _InterruptPropertyIndex, SignalID);
    
    _InterruptSignalID = 0;
}
