/* VmeAccess.hh */
/* VME interface class library */
/* Edited by Sanshiro E. on 6 October 1997. */
/* Last updated by Sanshiro E. on 16 April 1999. */

#ifndef __VmeAccess_hh__
#define __VmeAccess_hh__

#include <vector>
#include <sys/types.h>
#include "RoomDefs.hh"
#include "Module.hh"


enum TVmeTransferMode {
    tmA16D16, tmA16D32, 
    tmA24D16, tmA24D32, 
    tmA32D16, tmA32D32 
};

class TVmeController {
  public:
    TVmeController(void);
    virtual ~TVmeController();
    virtual int Open(TVmeTransferMode TransferMode) throw(THardwareException) = 0;
    virtual void Close(int DeviceDescripter) = 0;
    virtual caddr_t Map(int DeviceDescripter, off_t VmeAddress, size_t MapSize) throw(THardwareException) = 0;
    virtual void Unmap(caddr_t MappedAddress, size_t MapSize) throw(THardwareException) = 0;
    virtual void EnableInterrupt(int DeviceDesctipter, int InterruptNumber, int SignalID) throw(THardwareException) = 0;
    virtual void DisableInterrupt(int DeviceDesctipter, int InterruptNumber, int SignalID) throw(THardwareException) = 0;
};

class TVmeModule;

class TVmeCrate {
  public:
    TVmeCrate(TVmeController *VmeController, int NumberOfSlots = 21);
    TVmeCrate(int NumberOfSlots = 21);
    virtual ~TVmeCrate();
    void InstallController(TVmeController *VmeController);
    void Install(TVmeModule *VmeModule, off_t VmeAddress, int InterruptPropertyIndex = -1) throw(THardwareException);
    void Uninstall(TVmeModule *VmeModule);
  protected:
    TVmeController *_VmeController;
    vector<TVmeModule*> _ModuleList;
    int _NumberOfSlots;
};

class TVmeModule: public TModule {
  public:
    TVmeModule(size_t MapSize, TVmeTransferMode TransferMode);
    virtual ~TVmeModule();
    void EnableInterrupt(int SignalID) throw(THardwareException);
    void DisableInterrupt(void) throw(THardwareException);
    size_t MapSize(void) const;
    TVmeTransferMode TransferMode(void) const;
    off_t VmeAddress(void) const;
  protected:
    enum TWaitResult {wrOK, wrTimeOut};
    inline Word PutWord(off_t OffsetAddress, Word Value);
    inline Word GetWord(off_t OffsetAddress);
    inline Word *WordPtr(off_t OffsetAddress = 0);
    inline DoubleWord PutDoubleWord(off_t OffsetAddress, DoubleWord Value);
    inline DoubleWord GetDoubleWord(off_t OffsetAddress);
    inline DoubleWord *DoubleWordPtr(off_t OffsetAddress = 0);
    int WaitWord(off_t OffsetAddress, Word Value, unsigned TimeLimit = 0 /* sec */);
  private:
    friend void TVmeCrate::Install(TVmeModule *, off_t, int) throw(THardwareException);
    friend void TVmeCrate::Uninstall(TVmeModule *);
    struct TModuleCookie {
	TVmeController *VmeController;
	TVmeCrate *VmeCrate;
	off_t VmeAddress;
	int DeviceDescripter;
	volatile caddr_t MappedAddress;
	int InterruptPropertyIndex;
    };
    void Attach(const TModuleCookie &Cookie);
    void Detach(void);
    int DeviceDescripter(void) const;
    caddr_t MappedAddress(void) const;
  private:
    TVmeController *_VmeController;
    TVmeCrate *_VmeCrate;
    TVmeTransferMode _TransferMode;
    size_t _MapSize;
    int _DeviceDescripter;
    volatile caddr_t _MappedAddress;
    off_t _VmeAddress;
    int _IsAttached;
    int _InterruptPropertyIndex;
    int _InterruptSignalID;
};


inline Word TVmeModule::PutWord(off_t OffsetAddress, Word Value) {
    return *((Word *) (_MappedAddress + OffsetAddress)) = Value;
}

inline Word TVmeModule::GetWord(off_t OffsetAddress) {
    return *((Word *) (_MappedAddress + OffsetAddress));
}

inline Word *TVmeModule::WordPtr(off_t OffsetAddress) {
    return (Word *) (_MappedAddress + OffsetAddress);
}

inline DoubleWord TVmeModule::PutDoubleWord(off_t OffsetAddress, DoubleWord Value) {
    return *((DoubleWord *) (_MappedAddress + OffsetAddress)) = Value;
}

inline DoubleWord TVmeModule::GetDoubleWord(off_t OffsetAddress) {
    return *((DoubleWord *) (_MappedAddress + OffsetAddress));
}

inline DoubleWord *TVmeModule::DoubleWordPtr(off_t OffsetAddress) {
    return (DoubleWord *) (_MappedAddress + OffsetAddress);
}


#endif
