/* sample.cc */
/* ROOM sample program */
/* Edited by Sanshiro E. on 12 May 1999. */
/* Last updated by Sanshiro E. on 14 June 1999. */


#include <iostream.h>
#include "RoomDefs.hh"
#include "Controllers.hh"
#include "CamacModules.hh"
#include "DeviceFactory.hh"


// If your system can't handle interrupts, uncomment this line.
#define IRQ_IS_NOT_AVAILABLE


class TRoomSample {
  public:
    TRoomSample(void) throw(THardwareException);
    virtual ~TRoomSample();
    virtual int Run(int NumberOfEvents) throw(THardwareException);
  private:
    TCamacCrate* _Crate;
    TCamacController* _Controller;
    TCamacModule* _Adc;
};


TRoomSample::TRoomSample(void) throw(THardwareException)
{
    TDeviceFactory DeviceFactory;

    _Crate = DeviceFactory.CreateCamacCrate();
    _Controller = DeviceFactory.CreateCamacController("TOYO_CC7700");
    _Adc = DeviceFactory.CreateCamacModule("Rinei_RPC022");

    int StationNumber;
    _Controller->Initialize();
    _Crate->InstallController(_Controller);
    _Crate->Install(_Adc, StationNumber = 4);
}


TRoomSample::~TRoomSample()
{
    delete _Adc;
    delete _Controller;
    delete _Crate;
}


int TRoomSample::Run(int NumberOfEvents) throw(THardwareException)
{
    int EventCount = 0;
    int Channel, Data, Q;

    while (EventCount < NumberOfEvents) {
        
#ifndef IRQ_IS_NOT_AVAILABLE
        if (_Adc->WaitData()) {
#else
        while (! _Adc->HasData()); {
#endif
            for (Channel = 0; Channel < 3; Channel++) {
                Q = _Adc->Read(Channel, Data);
                cout << Data << endl;
                
                _Adc->Clear();
            }
            
            EventCount++;
        }
    }
    
    return EventCount;
}
    
int main(void)
{
    try {
        TRoomSample().Run(1024);
    }
    catch (THardwareException &e) {
        cerr << "ERROR: " << e << endl;
    }

    return 0;
}
